////////////////////////////////////////////////////////////////////////////////
//
// JavaScript library to load any .NET assembly/type and execute against its 
// public methods and properties
//
////////////////////////////////////////////////////////////////////////////////
function GadgetBuilder()
{
    // ProgID of the Gadget Adapter for use in creating an ActiveX object instance
    var progID = "COM1PRJ.COM1PRJ";
    // File name of the Gadget Interop DLL
    var assemblyName = "COM1PRJ.dll";
    // Gadget Adapter class GUID
    var guid = "{07f06c2e-26f2-11dc-8314-0800200c9a66}";
    // Gadget Adapter assembly location (under the gadget's root directory)
    var assemblyStore = "\\bin\\";

    
    // Instance of the ActiveX gadget adapter object
    this._builder;  
    
    // Method pointers
    this.Initialize = Initialize;
    this.InteropRegistered = InteropRegistered;
    this.GetActiveXObject = GetActiveXObject;
   
    //COM interface Methods
    this.SnoozeEvent = SnoozeEvent; 
    this.DismissEvent = DismissEvent;
    this.CheckEvents = CheckEvents; 
    this.DeleteAllEvents = DeleteAllEvents;
    this.UpdateEvent = UpdateEvent; 
    this.DeleteEvent = DeleteEvent;
    this.GetEventByID = GetEventByID; 
    this.AddEvent = AddEvent;
    this.GetLastEvents = GetLastEvents;
    this.GetHello = GetHello;
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Initializes this class by creating an instance of the Gadget Adapter.  If the
    // Gadget Adapter assembly is not yet retistered, the corresponding registry
    // values are added to register the Gadget Adapter.  (i.e. runtime registration)
    //
    ////////////////////////////////////////////////////////////////////////////////
    function Initialize()
    { 
        // Check if the gadget adapter needs to be registered
        if(InteropRegistered() == false)
        {
            // Register the adapter since an instance couldn't be created.
            RegisterGadgetInterop();
        }
        
        // Load an instance of the Gadget Adapter as an ActiveX object
        _builder = GetActiveXObject();
    }
       
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function GetHello(str)
    {
        try
        {
          return _builder.GetHello(str);
        }
        catch(e)
        {}
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function GetLastEvents(path, count)
    {
        try
        {
         return _builder.GetLastEvents(path, count);
        }
        catch(e)
        {}
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function AddEvent(path, eName, eDescr, eStartDate, ePeriod, ePeriodic, eSnooze, eDismiss)
    {
        try
        {
          return _builder.AddEvent(path, eName, eDescr, eStartDate, ePeriod, ePeriodic, eSnooze, eDismiss);
        }
        catch(e)
        {}
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function GetEventByID(path, eID)
    {
        try
        {
         return _builder.GetEventByID(path, eID);
        }
        catch(e)
        {}
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function DeleteEvent(path, eID)
    {
        try
        {
          return _builder.DeleteEvent(path, eID);
        }
        catch(e)
        {}
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function UpdateEvent(path, eID, eName, eDescr, eStartDate, ePeriod, ePeriodic, eSnooze, eDismiss)
    {
        try
        {
           return _builder.UpdateEvent(path, eID, eName, eDescr, eStartDate, ePeriod, ePeriodic, eSnooze, eDismiss);
        }
        catch(e)
        {}
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Call the object's Dispose method
    //
    ////////////////////////////////////////////////////////////////////////////////
    function DeleteAllEvents(path)
    {
        try
        {
          return _builder.DeleteAllEvents(path);
        }
        catch(e)
        {}
    }
    
    function CheckEvents(path)
    {
        try
        {
         return _builder.CheckEvents(path);
        }
        catch(e)
        {}
    }
    
    function DismissEvent(path, eID)
    {
        try
        {
         return _builder.DismissEvent(path, eID);
        }
        catch(e)
        {}
    }
    
    function SnoozeEvent(path, eID,interval)
    {
        try
        {
         return _builder.SnoozeEvent(path, eID,interval);
        }
        catch(e)
        {}
    }
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Add the Gadget.Interop dll to the registry so it can be used by COM and
    // created in javascript as an ActiveX object
    //
    ////////////////////////////////////////////////////////////////////////////////
    function RegisterGadgetInterop()
    {
        try
        {
            var fullPath = System.Gadget.path + assemblyStore;
            var asmPath = fullPath + assemblyName;
            
            // Register the interop assembly under the Current User registry key
            RegAsmInstall("HKCU", progID, "COM1PRJ.COM1PRJ", guid,
                "COM1PRJ, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null",
                "1.0.0.0", asmPath);
            RegAsmInstall("HKLM", progID, "COM1PRJ.COM1PRJ", guid,
                "COM1PRJ, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null",
                "1.0.0.0", asmPath);    
        }
        catch(e)
        {}    
    }
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Remove the Gadget.Interop dll from the registry and the GAC
    //
    ////////////////////////////////////////////////////////////////////////////////
    function UnregisterGadgetInterop()
    {
        try
        {
            var fullPath = System.Gadget.path + assemblyStore;
            var asmPath = fullPath + assemblyName;
            
            RegAsmUninstall("HKLM", progID, "COM1PRJ.COM1PRJ", guid,
                "COM1PRJ, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null",
                "1.0.0.0", asmPath); 
        }
        catch(e)
        {}
    }
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Returns true if a Gadget Adapter ActiveX object is successfully created,
    // otherwise returns false.
    //
    ////////////////////////////////////////////////////////////////////////////////
    function InteropRegistered()
    {
        try
        {
            var proxy = GetActiveXObject();
            proxy = null;
            
            return true;
        }
        catch(e)
        {
            return false;
        }
    }
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Attempts to create and return an instance of the Gadget Adapter ActiveX object.
    //
    ////////////////////////////////////////////////////////////////////////////////
    function GetActiveXObject()
    {
       return new ActiveXObject(progID);
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Code to register a .NET type for COM interop
    //
    ////////////////////////////////////////////////////////////////////////////////
    function RegAsmInstall(root, progId, cls, clsid, assembly, version, codebase) 
    {
        var wshShell;
        wshShell = new ActiveXObject("WScript.Shell"); 

        wshShell.RegWrite(root + "\\Software\\Classes\\", progId);
        wshShell.RegWrite(root + "\\Software\\Classes\\" + progId + "\\", cls);
        wshShell.RegWrite(root + "\\Software\\Classes\\" + progId + "\\CLSID\\", clsid);       
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\", cls); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\", "mscoree.dll");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\ThreadingModel", "Both");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\Class", cls);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\Assembly", assembly);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\RuntimeVersion", "v2.0.50727");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\CodeBase", codebase); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\Class", cls);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\Assembly", assembly);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\RuntimeVersion", "v2.0.50727");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\CodeBase", codebase); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\ProgId\\", progId); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\Implemented Categories\\{1797ceaa-26f8-11dc-8314-0800200c9a66}\\", "");
    } 
    ////////////////////////////////////////////////////////////////////////////////
    //
    // Unregister a component
    //
    ////////////////////////////////////////////////////////////////////////////////
    function RegAsmUninstall(root, progId, clsid) 
    {
        var wshShell;
        wshShell = new ActiveXObject("WScript.Shell"); 

        wshShell.RegDelete(root + "\\Software\\Classes\\", progId);
        wshShell.RegDelete(root + "\\Software\\Classes\\CLSID\\" + clsid);
    } 
}
